@@ -196,6 +196,12 @@ mreal mglBase::TextWidth(const char *text, const char *font, mreal size) const
 {	return (size<0?-size*FontSize:size)*font_factor*fnt->Width(text,(font&&*font)?font:FontDef)/20.16;	}
 mreal mglBase::TextWidth(const wchar_t *text, const char *font, mreal size) const
 {	return (size<0?-size*FontSize:size)*font_factor*fnt->Width(text,(font&&*font)?font:FontDef)/20.16;	}
+mreal mglBase::TextHeight(const char *text, const char *font, mreal size) const
+{	float y1,y2;	fnt->Width(text,(font&&*font)?font:FontDef,&y1,&y2);
+	return (size<0?-size*FontSize:size)*font_factor*(y2-y1)/20.16;	}
+mreal mglBase::TextHeight(const wchar_t *text, const char *font, mreal size) const
+{	float y1,y2;	fnt->Width(text,(font&&*font)?font:FontDef,&y1,&y2);
+	return (size<0?-size*FontSize:size)*font_factor*(y2-y1)/20.16;	}
 mreal mglBase::TextHeight(const char *font, mreal size) const
 {	return (size<0?-size*FontSize:size)*font_factor*fnt->Height(font?font:FontDef)/20.16; }
 void mglBase::AddActive(long k,int n)
@@ -1287,14 +1293,20 @@ char mglBase::SetPenPal(const char *p, long *Id, bool pal)
 }
 //-----------------------------------------------------------------------------
 // keep this for restore default mask
-MGL_EXPORT uint64_t mgl_mask_def[16]={0x000000FF00000000,	0x080808FF08080808,	0x0000FF00FF000000,	0x0000007700000000,
-							0x0000182424180000,	0x0000183C3C180000,	0x00003C24243C0000,	0x00003C3C3C3C0000,
-							0x0000060990600000,	0x0060584658600000,	0x00061A621A060000,	0x0000005F00000000,
-							0x0008142214080000,	0x00081C3E1C080000,	0x8142241818244281,	0x0000001824420000};
-MGL_EXPORT uint64_t mgl_mask_val[16]={0x000000FF00000000,	0x080808FF08080808,	0x0000FF00FF000000,	0x0000007700000000,
-							0x0000182424180000,	0x0000183C3C180000,	0x00003C24243C0000,	0x00003C3C3C3C0000,
-							0x0000060990600000,	0x0060584658600000,	0x00061A621A060000,	0x0000005F00000000,
-							0x0008142214080000,	0x00081C3E1C080000,	0x8142241818244281,	0x0000001824420000};
+MGL_EXPORT uint64_t mgl_mask_def[16]={
+	0x000000FF00000000,	0x080808FF08080808,	0x0000FF00FF000000,	0x0000000F00000000,
+	0x0000182424180000,	0x0000183C3C180000,	0x00003C24243C0000,	0x00003C3C3C3C0000,
+	0x0000060990600000,	0x0060584658600000,	0x00061A621A060000,	0x0000002700000000,
+	0x0008083E08080000,	0x0139010010931000,	0x0000001818000000,	0x101010FF010101FF};
+MGL_EXPORT uint64_t mgl_mask_val[16]={
+	0x000000FF00000000,	0x080808FF08080808,	0x0000FF00FF000000,	0x0000000F00000000,
+	0x0000182424180000,	0x0000183C3C180000,	0x00003C24243C0000,	0x00003C3C3C3C0000,
+	0x0000060990600000,	0x0060584658600000,	0x00061A621A060000,	0x0000002700000000,
+	0x0008083E08080000,	0x0139010010931000,	0x0000001818000000,	0x101010FF010101FF};
+// 	0x000000FF00000000,	0x080808FF08080808,	0x0000FF00FF000000,	0x0000007700000000,
+// 	0x0000182424180000,	0x0000183C3C180000,	0x00003C24243C0000,	0x00003C3C3C3C0000,
+// 	0x0000060990600000,	0x0060584658600000,	0x00061A621A060000,	0x0000005F00000000,
+//	0x0008142214080000,	0x00081C3E1C080000,	0x8142241818244281,	0x0000001824420000};
 void mglBase::SetMask(const char *p)
 {
 	mask = MGL_SOLID_MASK;	// reset to solid face
