@@ -302,8 +302,8 @@ std::string MGL_EXPORT mgl_str_arg(const std::string &str, char ch, int n1, int
 	{	p=str.find(ch,p+1);	pos.push_back(p?p+1:0);	}
 	std::string res;
 	if(n2<0)	n2=n1;
-	if(n1<0 || n1>=pos.size()-1 || n2<n1)	return res;
-	if(n2>=pos.size())	n2=pos.size()-1;
+	if(n1<0 || n1>=long(pos.size())-1 || n2<n1)	return res;
+	if(n2>=long(pos.size()))	n2=pos.size()-1;
 	res = str.substr(pos[n1],pos[n2+1]-pos[n1]-1);
 	if(res.size()==1 && res[0]==ch)	res.clear();
 	return res;
@@ -751,21 +751,21 @@ mreal MGL_EXPORT mgl_data_max_real(HCDT d, mreal *x, mreal *y, mreal *z)
 		if(im==0)	im=1;
 		if(im==nx-1)im=nx-2;
 		v1 = d->v(im+1,jm,km);	v2 = d->v(im-1,jm,km);
-		*x = (v1+v2-2*v)==0 ? im : im+(v1-v2)/(v1+v2-2*v)/2;
+		*x = (v1+v2-2*v)==0 ? im : im+(v2-v1)/(v1+v2-2*v)/2;
 	}
 	if(ny>2)
 	{
 		if(jm==0)	jm=1;
 		if(jm==ny-1)jm=ny-2;
 		v1 = d->v(im,jm+1,km);	v2 = d->v(im,jm-1,km);
-		*y = (v1+v2-2*v)==0 ? jm : jm+(v1-v2)/(v1+v2-2*v)/2;
+		*y = (v1+v2-2*v)==0 ? jm : jm+(v2-v1)/(v1+v2-2*v)/2;
 	}
 	if(nz>2)
 	{
 		if(km==0)	km=1;
 		if(km==nz-1)km=nz-2;
 		v1 = d->v(im,jm,km+1);	v2 = d->v(im,jm,km-1);
-		*z = (v1+v2-2*v)==0 ? km : km+(v1-v2)/(v1+v2-2*v)/2;
+		*z = (v1+v2-2*v)==0 ? km : km+(v2-v1)/(v1+v2-2*v)/2;
 	}
 	return m;
 }
@@ -785,21 +785,21 @@ mreal MGL_EXPORT mgl_data_min_real(HCDT d, mreal *x, mreal *y, mreal *z)
 		if(im==0)	im=1;
 		if(im==nx-1)im=nx-2;
 		v1 = d->v(im+1,jm,km);	v2 = d->v(im-1,jm,km);
-		*x = (v1+v2-2*v)==0 ? im : im+(v1-v2)/(v1+v2-2*v)/2;
+		*x = (v1+v2-2*v)==0 ? im : im+(v2-v1)/(v1+v2-2*v)/2;
 	}
 	if(ny>2)
 	{
 		if(jm==0)	jm=1;
 		if(jm==ny-1)jm=ny-2;
 		v1 = d->v(im,jm+1,km);	v2 = d->v(im,jm-1,km);
-		*y = (v1+v2-2*v)==0 ? jm : jm+(v1-v2)/(v1+v2-2*v)/2;
+		*y = (v1+v2-2*v)==0 ? jm : jm+(v2-v1)/(v1+v2-2*v)/2;
 	}
 	if(nz>2)
 	{
 		if(km==0)	km=1;
 		if(km==nz-1)km=nz-2;
 		v1 = d->v(im,jm,km+1);	v2 = d->v(im,jm,km-1);
-		*z = (v1+v2-2*v)==0 ? km : km+(v1-v2)/(v1+v2-2*v)/2;
+		*z = (v1+v2-2*v)==0 ? km : km+(v2-v1)/(v1+v2-2*v)/2;
 	}
 	return m;
 }
