@@ -300,21 +300,23 @@ dual mglFormulaC::CalcIn(const dual *a1) const
 	return NAN;
 }
 //-----------------------------------------------------------------------------
-mdual MGL_EXPORT_CONST mgl_ipowc(dual x,int n)
+dual MGL_LOCAL_CONST mgl_ipowc_c(dual x,int n)
 {
 	dual t;
 	if(n==2)	t = x*x;
 	else if(n==1)	t = x;
-	else if(n<0)	t = mreal(1)/mgl_ipowc(x,-n);
+	else if(n<0)	t = mreal(1)/mgl_ipowc_c(x,-n);
 	else if(n==0)	t = mreal(1);
 	else
 	{
-		t = mgl_ipowc(x,n/2);	t = t*t;
+		t = mgl_ipowc_c(x,n/2);	t = t*t;
 		if(n%2==1)	t *= x;
 	}
-	return t.real()+t.imag()*mgl_I;
+	return t;
 }
-mdual MGL_EXPORT mgl_ipowc_(dual *x,int *n)	{	return mgl_ipowc(*x,*n);	}
+cmdual MGL_EXPORT_CONST mgl_ipowc(mdual x,int n)
+{	return mdual(mgl_ipowc_c(x,n));	}
+cmdual MGL_EXPORT mgl_ipowc_(mdual *x,int *n)	{	return mgl_ipowc(*x,*n);	}
 //-----------------------------------------------------------------------------
 HAEX MGL_EXPORT mgl_create_cexpr(const char *expr)	{	return new mglFormulaC(expr);	}
 uintptr_t MGL_EXPORT mgl_create_cexpr_(const char *expr, int l)
@@ -323,10 +325,10 @@ uintptr_t MGL_EXPORT mgl_create_cexpr_(const char *expr, int l)
 	delete []s;	return res;	}
 void MGL_EXPORT mgl_delete_cexpr(HAEX ex)	{	if(ex)	delete ex;	}
 void MGL_EXPORT mgl_delete_cexpr_(uintptr_t *ex)	{	mgl_delete_cexpr((HAEX)ex);	}
-mdual MGL_EXPORT mgl_cexpr_eval(HAEX ex, dual x, dual y,dual z)
-{	dual r = ex->Calc(x,y,z);	return r.real()+r.imag()*mgl_I;	}
-mdual MGL_EXPORT mgl_cexpr_eval_(uintptr_t *ex, dual *x, dual *y, dual *z)
+cmdual MGL_EXPORT mgl_cexpr_eval(HAEX ex, mdual x, mdual y, mdual z)
+{	return mdual(ex->Calc(x,y,z));	}
+cmdual MGL_EXPORT mgl_cexpr_eval_(uintptr_t *ex, mdual *x, mdual *y, mdual *z)
 {	return mgl_cexpr_eval((HAEX) ex, *x,*y,*z);		}
-mdual MGL_EXPORT mgl_cexpr_eval_v(HAEX ex, dual *var)
-{	dual r = ex->Calc(var);	return r.real()+r.imag()*mgl_I;	}
+cmdual MGL_EXPORT mgl_cexpr_eval_v(HAEX ex, mdual *var)
+{	return mdual(ex->Calc(reinterpret_cast<dual*>(var)));	}
 //-----------------------------------------------------------------------------
